###################################################
### chunk number 1: 
###################################################
library( "micEconCES" )


###################################################
### chunk number 2: 
###################################################
set.seed( 123 )
cesData <- data.frame( x1 = rchisq( 200, 10 ), x2 = rchisq( 200, 10 ) )
cesData$y <- cesCalc( xNames = c( "x1", "x2" ), data = cesData,
   coef = c( gamma = 1, delta = 0.6, rho = 0.5, nu = 1.1 ) )
cesData$y <- cesData$y + 2.5 * rnorm( 200 )


###################################################
### chunk number 3: 
###################################################
cesNls <- nls( y ~ gamma * ( delta * x1^(-rho) + (1-delta) * x2^(-rho) )^(-phi/rho),
   data = cesData, start = c( gamma = 0.5, delta = 0.5, rho = 0.25, phi = 1 ) )
print( cesNls )


###################################################
### chunk number 4: 
###################################################
cesKmenta <- cesEst( yName = "y", xNames = c( "x1", "x2" ), data = cesData, 
   method = "Kmenta", vrs = TRUE )


###################################################
### chunk number 5: 
###################################################
summary( cesKmenta )


###################################################
### chunk number 6: 
###################################################
coef( summary( cesKmenta$kmenta ) )


###################################################
### chunk number 7: 
###################################################
pdf( "plotFittedKmenta.pdf", width = 4, height = 4 )
par( mar = c( 4.5, 4, 1, 1 ) )


###################################################
### chunk number 8: 
###################################################
compPlot ( cesData$y, fitted( cesKmenta ), xlab = "actual values", 
   ylab = "fitted values" )


###################################################
### chunk number 9: 
###################################################
dev.off()


###################################################
### chunk number 10: 
###################################################
cesLm <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE )
summary( cesLm )


###################################################
### chunk number 11: 
###################################################
pdf( "plotFittedLm.pdf", width = 4, height = 4 )
par( mar = c( 4.5, 4, 1, 1 ) )


###################################################
### chunk number 12: 
###################################################
compPlot ( cesData$y, fitted( cesLm ), xlab = "actual values", 
   ylab = "fitted values" )


###################################################
### chunk number 13: 
###################################################
dev.off()


###################################################
### chunk number 14: 
###################################################
cesCg <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "CG" )
summary( cesCg )


###################################################
### chunk number 15: 
###################################################
cesCg2 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "CG",
   control = list( maxit = 1000, reltol = 1e-5 ) )
summary( cesCg2 )


###################################################
### chunk number 16: 
###################################################
cesNewton <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, 
   method = "Newton" )
summary( cesNewton )


###################################################
### chunk number 17: 
###################################################
cesBfgs <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "BFGS" )
summary( cesBfgs )


###################################################
### chunk number 18: 
###################################################
cesNm <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, 
   method = "NM" )
summary( cesNm )


###################################################
### chunk number 19: 
###################################################
cesSann <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN" )
summary( cesSann )


###################################################
### chunk number 20: 
###################################################
cesSann2 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN" )
all.equal( cesSann, cesSann2 )


###################################################
### chunk number 21: 
###################################################
cesSann3 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN",
   random.seed = 1234 )
cesSann4 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN",
   random.seed = 12345 )
cesSann5 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN",
   random.seed = 123456 )
m <- rbind( cesSann = coef( cesSann ), cesSann3 = coef( cesSann3 ),
   cesSann4 = coef( cesSann4 ), cesSann5 = coef( cesSann5 ) )
rbind( m, stdDev = sd( m ) )


###################################################
### chunk number 22: 
###################################################
cesSannB <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN",
   control = list( maxit = 100000 ) )
cesSannB3 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN",
   random.seed = 1234, control = list( maxit = 100000 ) )
cesSannB4 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN",
   random.seed = 12345, control = list( maxit = 100000 ) )
cesSannB5 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "SANN",
   random.seed = 123456, control = list( maxit = 100000 ) )
m <- rbind( cesSannB = coef( cesSannB ), cesSannB3 = coef( cesSannB3 ),
   cesSannB4 = coef( cesSannB4 ), cesSannB5 = coef( cesSannB5 ) )
rbind( m, stdDev = sd( m ) )


###################################################
### chunk number 23: 
###################################################
cesDe <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   control = list( trace = FALSE ) )
summary( cesDe )


###################################################
### chunk number 24: 
###################################################
cesDe2 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   control = list( trace = FALSE ) )
all.equal( cesDe, cesDe2 )


###################################################
### chunk number 25: 
###################################################
cesDe3 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   random.seed = 1234, control = list( trace = FALSE ) )
cesDe4 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   random.seed = 12345, control = list( trace = FALSE ) )
cesDe5 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   random.seed = 123456, control = list( trace = FALSE ) )
m <- rbind( cesDe = coef( cesDe ), cesDe3 = coef( cesDe3 ),
   cesDe4 = coef( cesDe4 ), cesDe5 = coef( cesDe5 ) )
rbind( m, stdDev = sd( m ) )


###################################################
### chunk number 26: 
###################################################
cesDeB <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   control = list( trace = FALSE, itermax = 1000 ) )
cesDeB3 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   random.seed = 1234, control = list( trace = FALSE, itermax = 1000 ) )
cesDeB4 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   random.seed = 12345, control = list( trace = FALSE, itermax = 1000 ) )
cesDeB5 <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, method = "DE",
   random.seed = 123456, control = list( trace = FALSE, itermax = 1000 ) )
rbind( cesDeB = coef( cesDeB ), cesDeB3 = coef( cesDeB3 ),
   cesDeB4 = coef( cesDeB4 ), cesDeB5 = coef( cesDeB5 ) )


###################################################
### chunk number 27: 
###################################################
cesLbfgsb <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, 
   method = "L-BFGS-B" )
summary( cesLbfgsb )


###################################################
### chunk number 28: 
###################################################
cesPort <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, 
   method = "PORT" )
summary( cesPort )


###################################################
### chunk number 29: 
###################################################
cesGrid <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, 
   rho = seq( from = -0.3, to = 1.5, by = 0.1 ) )
summary( cesGrid )


###################################################
### chunk number 30: 
###################################################
pdf( "plotGrid.pdf", width = 4, height = 4 )
par( mar = c( 4.5, 4, 1, 1 ) )


###################################################
### chunk number 31: 
###################################################
plot( cesGrid )


###################################################
### chunk number 32: 
###################################################
dev.off()


###################################################
### chunk number 33: 
###################################################
cesStartGrid <- cesEst( "y", c( "x1", "x2" ), cesData, vrs = TRUE, 
   start = coef( cesGrid ) )
summary( cesStartGrid )


###################################################
### chunk number 34: 
###################################################
pdf( "plotCesCalcRho.pdf", width = 4.2, height = 4 )
par( mar = c( 4.5, 4, 1.5, 1 ) )


###################################################
### chunk number 35: 
###################################################
rhoData <- data.frame( rho = seq( -2e-6, 2e-6, 5e-9 ),
   yCES = NA, yLin = NA )
# calculate endogenous variables
for( i in 1:nrow( rhoData ) ) {
   # vector of coefficients
   cesCoef <- c( gamma = 1, delta = 0.6, rho = rhoData$rho[ i ], nu = 1.1 )
   rhoData$yLin[ i ] <- cesCalc( xNames = c( "x1", "x2" ), data = cesData[1,],
      coef = cesCoef, rhoApprox = Inf )
   rhoData$yCES[ i ] <- cesCalc( xNames = c( "x1", "x2" ), data = cesData[1,],
      coef = cesCoef, rhoApprox = 0 )
}
# normalise output variables
rhoData$yCES <- rhoData$yCES - rhoData$yLin[ rhoData$rho == 0 ]
rhoData$yLin <- rhoData$yLin - rhoData$yLin[ rhoData$rho == 0 ]
plot( rhoData$rho, rhoData$yCES, type = "l", col = "red",
   xlab = "rho", ylab = "y (normalised, red = CES, black = linearised)" )
lines( rhoData$rho, rhoData$yLin )


###################################################
### chunk number 36: 
###################################################
dev.off()


###################################################
### chunk number 37: 
###################################################
pdf( "plotCesCalcRho2.pdf", width = 4.2, height = 4 )
par( mar = c( 4.5, 4, 1.5, 1 ) )
rhoData <- data.frame( rho = seq( -1, 3, 5e-2 ),
   yCES = NA, yLin = NA )
# calculate endogenous variables
for( i in 1:nrow( rhoData ) ) {
   # vector of coefficients
   cesCoef <- c( gamma = 1, delta = 0.6, rho = rhoData$rho[ i ], nu = 1.1 )
   rhoData$yLin[ i ] <- cesCalc( xNames = c( "x1", "x2" ), data = cesData[1,],
      coef = cesCoef, rhoApprox = Inf )
   rhoData$yCES[ i ] <- cesCalc( xNames = c( "x1", "x2" ), data = cesData[1,],
      coef = cesCoef, rhoApprox = 0 )
}
# normalise output variables
rhoData$yCES <- rhoData$yCES - rhoData$yLin[ rhoData$rho == 0 ]
rhoData$yLin <- rhoData$yLin - rhoData$yLin[ rhoData$rho == 0 ]
plot( rhoData$rho, rhoData$yCES, type = "l", col = "red",
   xlab = "rho", ylab = "y (normalised, red = CES, black = linearised)" )
lines( rhoData$rho, rhoData$yLin )
dev.off()


